//+----------------------------------------------------------------------------+
//|                               mp6140 GBPUSD Daily Hedge EA for MetaTrader4 |
//|                                                                version 3.2 |
//|                                                  original idea from mp6140 |
//|                                                     EA conversion by azmel |
//+----------------------------------------------------------------------------+
//| VERSION HISTORY                                                            |
//| 1   Initial version. Settings for three AccountTypes. New trades won't     |
//|     open until previous trades closes.                                     |
//| 2   Fixed SELL bug                                                         |
//| 3   Added Money Management and Take Profit. New hedge opens regardless of  |
//|     previous hedge closed or not.                                          |
//| 3.1 Hedge can be opened by the minute instead of by the hour.              | 
//| 3.2 Added UseBreakEven feature and customisable buy/sell comments.         |
//|     TimeToTrade entry is now entered with a four-character text.           |
//| 3.31 Added 7 TimeToTrade at one chart, init section                        |               
//+----------------------------------------------------------------------------+
//| INSTRUCTIONS                                                               |
//| To be used on GBPUSD pair only with any timeframe. EA will place a hegde   |
//| (a BUY and a SELL simultaneously) at the specified time. Break Even        |
//| feature is introduced whereby if one of the trade hit its TakeProfit's     |
//| value, the other trade will move to Break Even, to minimise risk.          |
//+----------------------------------------------------------------------------+
//| SETTINGS                                                                   |
//| AccountType=0 for Standard (1 Lot) Accounts                                |
//| AccountType=1 for Mini (0.1 Lot) Accounts                                  |
//| AccountType=2 for Micro (0.01 Lot) Accounts                                |
//| AccountType=3 for InterbankFX Nano (0.01 Lot) Accounts                     |
//+----------------------------------------------------------------------------+

#include <stdlib.mqh>

//+----------------------------------------------------------------------------+
//| External Variables                                                         |
//+----------------------------------------------------------------------------+

extern int    Magic             = 54281;
extern string TimeToTrade_1     = "1445";
extern string TimeToTrade_2     = "1615";
extern string TimeToTrade_3     = "1745";
extern string TimeToTrade_4     = "2115";
extern string TimeToTrade_5     = "0015";
extern string TimeToTrade_6     = "0245";
extern string TimeToTrade_7     = "0415";

extern double Lots              = 0.01;
extern int    TakeProfit        = 10;
extern int    StopLoss          = 0;
extern bool   UseStopLoss       = false;
extern bool   MoneyManagement   = false;
extern int    PercentageToTrade = 40;
extern int    AccountType       = 2;
extern int    Leverage          = 0;
extern bool   UseBreakEven      = true;
extern string BuyComment        = "mp6140-buy";
extern string SellComment       = "mp6140-sell";

//+----------------------------------------------------------------------------+
//| Internal Variables                                                         |
//+----------------------------------------------------------------------------+

double   MaxLots;
double   DecimalCorrection;
int      DecimalPlaces;
double   PricePerPip;

int      HourToTrade[6];
int      MinuteToTrade[6];

bool     OKToTrade;
int      Status [6]={1,1,1,1,1,1,1};
int      BuyFlag[6];
int      SellFlag[6];
int      SecureBuyFlag[6];
int      SecureSellFlag[6];
int      AccLeverage;

int      ticket1[6];
int      ticket2[6];
int      ticket3[6];
double   TP1[6];
double   TP2[6];
double   TP3;
double   L3;
int      err;
int      slippage=5;
int      y;

string TimeToTrade;

// Completed Phases (for reference only):
// 0 - No trade allowed.
// 1 - Standby and waiting.
// 2 - Buy and Sell Placed.
// 3 - Buy or Sell Moved to Break Even.
// 4 - (Reserved for future development).

//+----------------------------------------------------------------------------+
//| init                                                                       |
//+----------------------------------------------------------------------------+
int init()

{

   if (Leverage==0)AccLeverage=AccountLeverage();
   else AccLeverage=Leverage;
    
   if (AccountType==0)
   {
      PricePerPip       = 10;
      DecimalCorrection = 0.5;
      DecimalPlaces     = 0;
   }
   
   if (AccountType==1)
   { 
      PricePerPip       = 10;
      DecimalCorrection = 0.05;
      DecimalPlaces     = 1;
   }
   
   if (AccountType==2)
   {
      PricePerPip       = 10;
      DecimalCorrection = 0.005;
      DecimalPlaces     = 2;
   }
   
   if (AccountType==3)
   {
      PricePerPip       = 1;
      DecimalCorrection = 0.005;
      DecimalPlaces     = 2;
   }
   
}


//+----------------------------------------------------------------------------+
//| Start                                                                      |
//+----------------------------------------------------------------------------+
int start()

{
   for ( y = 0; y < 6; y++)
   {

      if (MoneyManagement)
      {
         if (PercentageToTrade > 50)
         {
            Comment("ERROR: PercentageToTrade value too large.");
            OKToTrade=false;
         }
         else
         {
            if (PercentageToTrade < 1)
            {
               Comment("ERROR: PercentageToTrade value too small.");
               OKToTrade=false;
            }
            else
            {
               Lots=NormalizeDouble(((AccountFreeMargin()*PercentageToTrade*AccLeverage)/
                  (Ask*PricePerPip*1000000))-DecimalCorrection,DecimalPlaces);
               OKToTrade=true;
            }
         }
      }
      else
      {
         MaxLots=NormalizeDouble(((AccountFreeMargin()*50*AccLeverage)/
            (Ask*PricePerPip*1000000))-DecimalCorrection,DecimalPlaces);
         if (Lots > MaxLots)
         {
            Comment("ERROR: Insufficient Free Margin.\n"
               ,"Lower Lots value or use MoneyManagement.");
            OKToTrade=false;
         }
         else
         {
            OKToTrade=true;
         }
      }




      if (y==0)TimeToTrade=TimeToTrade_1;
      if (y==1)TimeToTrade=TimeToTrade_2;
      if (y==2)TimeToTrade=TimeToTrade_3;
      if (y==3)TimeToTrade=TimeToTrade_4;
      if (y==4)TimeToTrade=TimeToTrade_5;
      if (y==5)TimeToTrade=TimeToTrade_6;
      if (y==6)TimeToTrade=TimeToTrade_7;

      
      if (StringLen(TimeToTrade)==4)
      { 
      
      
         HourToTrade[y]=StrToInteger(StringSubstr(TimeToTrade,0,2));
         MinuteToTrade[y]=StrToInteger(StringSubstr(TimeToTrade,2,2));
         if (HourToTrade[y]<0 || HourToTrade[y]>23 || MinuteToTrade[y]<0 || MinuteToTrade[y]>59)
         {
            Comment("Hedge Time Error.");
            OKToTrade=false;
         }
      }
      else
      {
         Comment("Hedge Time Error.");
         OKToTrade=false;
      }  




      if (OKToTrade && Status[y]==1 && Hour()==HourToTrade[y] && Minute()==MinuteToTrade[y])
      {
         if (BuyFlag[y]==0)
         {
            ticket1[y]=OrderSend(Symbol(),OP_BUY,Lots,Ask,slippage
               ,(Ask-StopLoss*Point)*UseStopLoss,Ask+TakeProfit*Point,BuyComment
               ,Magic,Blue);
            if (ticket1[y]== -1)
            {
               Print("BUY ERROR: ",ErrorDescription(GetLastError())," at ", Hour()
                  ,":",Minute());
            }
            else
            {
               OrderSelect(ticket1[y],SELECT_BY_TICKET);
               TP1[y]=OrderTakeProfit();         
               BuyFlag[y]=1;
            }
         }


         if (SellFlag[y]==0)
         {
            ticket2[y]=OrderSend(Symbol(),OP_SELL,Lots,Bid,slippage
               ,(Bid+StopLoss*Point)*UseStopLoss,Bid-TakeProfit*Point,SellComment
               ,Magic,Red);
            if (ticket2[y] == -1)
            {
               Print("SELL ERROR: ",ErrorDescription(GetLastError())," at ", Hour()
                  ,":",Minute());
            }
            else
            {
               OrderSelect(ticket2[y],SELECT_BY_TICKET);
               TP2[y]=OrderTakeProfit();         
               SellFlag[y]=1;
            }
         }

         if (BuyFlag[y]==1 && SellFlag[y]==1)
         {
            Status[y]=2;
         }
      }

      if (Status[y]==2 && UseBreakEven)
      {
         if (Bid >= TP1[y] && SecureSellFlag[y]==0)
         {
            OrderSelect(ticket2[y],SELECT_BY_TICKET);
            TP3=OrderOpenPrice();
            L3=OrderLots();
            if (OrderModify(ticket2[y],OrderOpenPrice(),OrderStopLoss(),TP3,0,CLR_NONE))
            {
               SecureSellFlag[y]=1;
            }
            else
            {
               Print("SECURE SELL ERROR: ",ErrorDescription(GetLastError())," at "
                  , Hour(),":",Minute());            
            }
         }

         if (Ask <= TP2[y] && SecureBuyFlag[y]==0)
         {
            OrderSelect(ticket1[y],SELECT_BY_TICKET);
            TP3=OrderOpenPrice();
            L3=OrderLots();
            if (OrderModify(ticket1[y],OrderOpenPrice(),OrderStopLoss(),TP3,0,CLR_NONE))
            {
               SecureBuyFlag[y]=1;
            }
            else
            {
               Print("SECURE BUY ERROR: ",ErrorDescription(GetLastError())," at "
                  , Hour(),":",Minute());            
            }
         }
      
         if (SecureSellFlag[y] || SecureBuyFlag[y])
         {
            Status[y]=3;
         }
      }

      if (Status[y]==2 && !UseBreakEven)   
      {
         Status[y]=3;
      }

      //+-------------------------------------------------------------------------+
      //| Status 3 is reserved for future development                             |
      //+-------------------------------------------------------------------------+

      if (Status[y]==3)
      {
         Status[y]=4;
         BuyFlag[y]=0;
         SellFlag[y]=0;
         SecureBuyFlag[y]=0;
         SecureSellFlag[y]=0;
      }

      if (Status[y]==4 && Hour() != HourToTrade[y])
      {
         Status[y]=1;
      }


/*      if (OKToTrade)
      {
         if (MinuteToTrade[y]<10)
         {
            Comment("AccountType = ",AccountType,"\nLots = ",Lots,"\nHedge time = ",HourToTrade[y],":0",MinuteToTrade[y]);
         }
         else
         {
            Comment("AccountType = ",AccountType,"\nLots = ",Lots,"\nHedge time = ",HourToTrade[y],":",MinuteToTrade[y]);
         }
      }
*/

   }
   return(0);

}